<?php
/*******************************************************************************
 @name Bluepay
 @type Merchant Gateway
 @author Cody Phillips
 @date May 7, 2009
 @version 1.3
 @notes For use with Blesta version 2.2.0+
*******************************************************************************/

class bluepay extends Gateway {
	private $bluepay_url = "https://secure.bluepay.com/interfaces/bp20post";
	private $curl = "/usr/bin/curl";
	private $username;
	private $key;
	private $transType = "SALE";
	private $testmode = "TEST";
	private $vars;
	private $data;
	private static $settingFields = array("username" => "Account ID", "key" => "Secret Key", "testmode" => "Test Mode");
	
	// Note: Bluepay.com supports the following currency codes.  They may also support many others not listed here.
	// for a complete list see Bluepay's API documentation.
	private static $supportedCurrencyCodes = array("USD");


	/*
	*  Initializes a bluepay gateway
	*/
	public function __construct($gatewayinfo) {
		$this->username = $gatewayinfo['username']['value'];
		$this->key = $gatewayinfo['key']['value'];
		$this->testmode = ($gatewayinfo['testmode']['value'] == "True" ? "TEST" : "LIVE");
	}
	 
	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}


	/***
	* calcTPS()
	*
	* Calculates & returns the tamper proof seal md5.
	*/
	private final function calcTPS() {
		/*
		$hashstr = $this->secretKey . $this->accountId . $this->transType .
		$this->amount . $this->masterId . $this->name1 . $this->account;
		*/

		$hashstr = $this->key . $this->username . $this->transType .
		$this->vars['amount'] . $this->vars['fname'] . $this->vars['ccno'];

		return bin2hex(md5($hashstr, true));
		//return md5($hashstr, true);
	}


	/***
	* process()
	*
	* Will first generate the tamper proof seal, then
	* populate the POST query, then send it, and store
	* the response, and finally parse the response.
	*/
	public function processCard() {
		/* calculate the tamper proof seal */
		$tps = $this->calcTPS();
		$tpsdef = "ACCOUNT_ID TRANS_TYPE AMOUNT NAME1 PAYMENT_ACCOUNT";
		/* fill in the fields */
		$fields = array (
		'ACCOUNT_ID' => $this->username,
		//'USER_ID' => $this->userId,
		'TPS_DEF' => $tpsdef,
		'TAMPER_PROOF_SEAL' => $tps,
		'TRANS_TYPE' => $this->transType,
		'PAYMENT_TYPE' => "CREDIT",
		'MODE' => $this->testmode,
		'MASTER_ID' => $this->masterId,

		'PAYMENT_ACCOUNT' => $this->vars['ccno'],
		//'CARD_CVV2' => $this->vars['cvv2'],
		'CARD_EXPIRE' => $this->vars['ccexp'],
		//'SSN' => $this->ssn,
		//'BIRTHDATE' => $this->birthdate,
		'CUST_ID' => $this->vars['cid'],
		//'CUST_ID_STATE' => $this->custIdState,
		'AMOUNT' => $this->vars['amount'],
		'NAME1' => $this->vars['fname'],
		'NAME2' => $this->vars['lname'],
		'ADDR1' => $this->vars['address'],
		//'ADDR2' => $this->addr2,
		'CITY' => $this->vars['city'],
		'STATE' => $this->vars['state'],
		'ZIP' => $this->vars['zip'],
		'COUNTRY' => $this->vars['country'],
		'MEMO' => $this->vars['description'],
		//'ORDER_ID' => $this->orderId,
		'INVOICE_ID' => $this->vars['iid'],
		//'AMOUNT_TIP' => $this->tip,
		//'AMOUNT_TAX' => $this->tax,

		//'DO_REBILL' => $this->doRebill,
		//'REB_FIRST_DATE' => $this->rebDate,
		//'REB_EXPR' => $this->rebExpr,
		//'REB_CYCLES' => $this->rebCycles,
		//'REB_AMOUNT' => $this->rebAmount,

		//'DO_AUTOCAP' => $this->doAutocap,
		//'AVS_ALLOWED' => $this->avsAllowed,
		//'CVV2_ALLOWED' => $this->cvv2Allowed
		);

		/* perform the transaction */
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $this->bluepay_url); // Set the URL
		curl_setopt($ch, CURLOPT_USERAGENT, "Blesta"); // Cosmetic
		curl_setopt($ch, CURLOPT_POST, 1); // Perform a POST
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); // If not set, curl prints output to the browser
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($fields));

		// Log what we send to Bluepay
		$this->logData("bluepay", str_replace($this->vars['ccno'], "xxxx", http_build_query($fields)), "input", true);           
		
		$response = curl_exec($ch);
		curl_close($ch);

		// return the parsed response
		return $this->parseResponse($response);
	}

	/***
	* parseResponse()
	*
	* This method will parse the response parameter values
	* into the respective properties.
	*/
	private function parseResponse($response) {
		parse_str($response, $out);

		// Takes the results from bluepay and builds an array with the correct key value pairs
		$bluepay_results=array(
		"x_response_code"=> ($out['STATUS'] == 'E' ? 3 : ($out['STATUS'] == 1 ? 1 : 2)),
		"x_response_reason_text"=> $out['MESSAGE'],
		"x_auth_code"=> $out['AUTH_CODE'],
		"x_trans_id"=> $out['TRANS_ID']);

		// Log what we receive from Bluepay
		$this->logData("bluepay", $response, "output", ($out['STATUS'] == 1 ? true : false));           
		
		return $bluepay_results;
	}
}



/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
VISA                4007 0000 0002 7
                    4111 1111 1111 1111
MASTERCARD          5454 5454 5454 5454
American Express    3434 3434 3434 343
Discover            6011 6011 6011 6611
JCB                 3088 0000 0000 0009
Diner's Club        3000 0000 0000 04
ACH                 052000113 (routing number)
                    123123123 (account number)

*/
?>