<?php
/*******************************************************************************
 @name Website Payments Pro
 @type Merchant Gateway
 @author Cody Phillips
 @date August 19, 2009  
 @version 1.0
 @notes For use with Blesta version 2.2.0+
 
 This implementation covers the Name-Value Pair process using an API signature.
 
*******************************************************************************/

class websitepaymentspro extends Gateway {
	private $api_version = "51.0"; // The version of Paypal's API to use
	private $paypay_url = "https://api-3t.paypal.com/nvp/"; // Live mode;
	private $paypal_url_test = "https://api-3t.sandbox.paypal.com/nvp/"; // Test mode;
	private $username;
	private $key;
	private $signature;
	private $vars;
	
	private static $countries = null;
	
	private static $settingFields = array("username" => "API Username", "key" => "API Password", "opt1" => "Signature", "testmode" => "Test Mode");

	// Note: Paypal Website Payments Pro supports the following currency codes.
	// They may also support many others not listed here. For a complete listing see Paypal.com
	private static $supportedCurrencyCodes = array("AUD", "CAD", "CHF", "CZK", "DKK", "EUR", "GBP",
		"HKD", "HUF", "ILS", "JPY", "MXN", "NOK", "NZD", "PLN", "SGD", "SEK", "USD");

	function __construct($gatewayinfo) {
		$this->username = $gatewayinfo['username']['value'];
		$this->key = $gatewayinfo['key']['value'];
		$this->signature = $gatewayinfo['opt1']['value'];
		
		if ($gatewayinfo['testmode']['value'] == "True")
			$this->paypay_url = $this->paypal_url_test;

		$this->currency = (in_array($gatewayinfo['currency'], self::$supportedCurrencyCodes) ? $gatewayinfo['currency'] : "");
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}
	
	public function getSettingFields() {
		return self::$settingFields;	
	}

	/*
	*  We use this to set all the vars we will be using like fname, lname, phone, etc
	*/
	public function set($name, $value) {
		$this->vars[$name] = $value;
	}

	public function processCard() {
		// Fetch all available countries, since we'll need to send paypal a 2-char
		// country code, instead of the usual 3-char.
		if (self::$countries == null) {
			global $mysqldb;
			$temp = getcountries($mysqldb);
			for ($i=0; $i<count($temp); $i++)
				self::$countries[$temp[$i]['3char']] = $temp[$i];
		}
		
		$cc_month = substr($this->vars['ccexp'], 0, 2);
		$cc_year = substr(date("Y"), 0, 2) . substr($this->vars['ccexp'], 2, 2);
		
		$data = array();
		$data['METHOD'] = "DoDirectPayment";
		$data['VERSION'] = $this->api_version;
		$data['PWD'] = $this->key;
		$data['USER'] = $this->username;
		$data['SIGNATURE'] = $this->signature;
		$data['PAYMENTACTION'] = "Sale";
		$data['AMT'] = $this->vars['amount'];
		$data['CREDITCARDTYPE'] = $this->getCardType($this->vars['ccno']);
		$data['ACCT'] = $this->vars['ccno'];
		$data['EXPDATE'] = $cc_month . $cc_year;
		$data['CVV2'] = "";
		$data['FIRSTNAME'] = $this->vars['fname'];
		$data['LASTNAME'] = $this->vars['lname'];
		$data['STREET'] = $this->vars['address'];
		$data['STREET2'] = $this->vars['address2'];
		$data['CITY'] = $this->vars['city'];
		$data['STATE'] = $this->vars['state'];
		$data['ZIP'] = $this->vars['zip'];
		$data['COUNTRYCODE'] = self::$countries[$this->vars['country']]['2char'];
		$data['CURRENCYCODE'] = $this->currency;
		$data['DESC'] = $this->vars['description'];
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $this->paypay_url);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

		// Log what we sent to paypal
		$this->logData("websitepaymentspro", str_replace($this->vars['ccno'], "xxxx", $data), "input", true);

		$response = curl_exec($ch);

		curl_close($ch);
		
		// Return the parsed response back to Blesta
		return $this->parseResponse($response);
	}

	/**
	 * This method will parse the response parameter values
	 * into the respective properties.
	 *
	 * @param string $response A string containing the output of an HTTP request
	 * @return array An array containing the parsed response ready for Blesta
	 */
	private function parseResponse($response) {
		parse_str($response, $out);

		// Takes the results from paypal and builds an array with the correct key value pairs
		$paypal_results=array(
		"x_response_code"=> ($out['ACK'] == "Success" || $out['ACK'] == "SuccessWithWarning" ? 1 : (isset($out['L_SHORTMESSAGE0']) ? 3 : 2)),
		"x_response_reason_text"=> (isset($out['L_LONGMESSAGE0']) ? $out['L_LONGMESSAGE0'] : "Success"),
		"x_trans_id"=> $out['TRANSACTIONID']);

		// Log what we receive from paypal
		$this->logData("websitepaymentspro", $response, "output", ($paypal_results['x_response_code'] == 1 ? true : false));

		return $paypal_results;
	}
	
	
	/**
	 * This method returns the card type associated with the number, if it can
	 * be deteremined, otherwise false is returned.
	 *
	 * @param string $cardnum The card number to evaluate
	 * @return mixed false if $cardnum is not recognized, string as the card type otherwise
	 */
	private function getCardType($cardnum) {
		$pattern[0] = "/^([4]{1})([0-9]{12,15})$/"; // Visa
		$pattern[1] = "/^([51|52|53|54|55]{2})([0-9]{14})$/"; // Mastercard
		$pattern[2] = "/^(5020|5038|5641|6759)([0-9]{12})$/"; // Maestro
		$pattern[3] = "/^([30|36|38]{2})([0-9]{12})$/"; // Diner's Club
		$pattern[4] = "/^([34|37]{2})([0-9]{13})$/"; // American Express
		$pattern[5] = "/^(3[0-9]{15})$|^(2131|1800[0-9]{11})$/"; // JCB
		$pattern[6] = "/^6334[5-9]([0-9]{11}|[0-9]{13,14})|^6767([0-9]{12}|[0-9]{14,15})$/"; // Solo
		$pattern[7] = "/^6(?:011|5[0-9]{2})[0-9]{12}$/"; // Discover
		
		$returnval = array("Visa", "MasterCard", "Maestro", "DinersClub", "Amex", "JCB", "Solo", "Discover");
		
		for ($i=0; $i<count($pattern); $i++) {
			if (preg_match($pattern[$i], $cardnum))
				return $returnval[$i];
		}
		
		return false;
	}
}

/*
--------------------------------------------
Test Account Numbers
--------------------------------------------
American Express    378282246310005
American Express    371449635398431
Amex Corporate      378734493671000
Australian BankCard 5610591081018250
Diners Club         30569309025904
Diners Club         38520000023237
Discover            6011111111111117
Discover            6011000990139424
JCB                 3530111333300000
JCB                 3566002020360505
MasterCard          5555555555554444
MasterCard          5105105105105100
Visa                4111111111111111
Visa                4012888888881881
Visa                4222222222222
*/
?>