<?php
/*******************************************************************************
 @name Google Checkout
 @type Non-Merchant Gateway
 @author Cody Phillips
 @date Sep 14, 2010
 @version 1.2
 @notes For use with Blesta version 2.2.0+

 In order to use Google Checkout you must have an SSL certificate.

 Login to your Google Checkout account, then click Settings -> Preferences ->
 Order processing preferences and select to "Automatically authorize and charge
 the buyer's credit card."

 In Settings -> Integration be sure to set the callback url to:
 http://www.yourdomain.com/callback.php?gw=googlecheckout&key={YOUR CALLBACK-KEY}

 Also Select HTML (name/value pairs) as the Callback method.

 {YOUR CALLBACK-KEY} is the key you set in Blesta for the Callback-Key for
 this gateway.  It can be anything you like, but if you leave it empty you will
 risk spoof payment attempts from people pretending to be Google.

*******************************************************************************/

class googlecheckout extends Gateway {
	private $google_url = "https://checkout.google.com/cws/v2/Merchant/"; // Live mode;
	private $google_url_test = "https://sandbox.google.com/checkout/cws/v2/Merchant/"; // Test mode;
	private $testmode = "True";
	private $currency;
	private $gatewayinfo;
	private $orderinfo;
	private $button = array("url" => "https://sandbox.google.com/checkout/buttons/checkout.gif?merchant_id=", "alt" => "Fast checkout through Google");
	private $hiddenFields = array("cart" => "", "signature" => "");
	private static $settingFields = array("username" => "Merchant-ID", "key" => "Merchant-Key", "opt1" => "Callback-Key", "testmode" => "Test Mode");
	private static $gw = "googlecheckout"; // Must be defined in /inc/config.php under "PAYTYPES"

	// Note: Google Checkout supports the following currency codes.
	// If the currency code being used by Blesta matches one of the following it will be sent to the payment gateway.
	// 1 = on, 0 = off
	private static $supportedCurrencyCodes = array("USD", "GBP");

	function __construct($gatewayinfo, $orderinfo) {
		$this->gatewayinfo = $gatewayinfo;
		$this->orderinfo = $orderinfo;

		//$currency = (googlecheckout::$supportedCurrencyCodes[$gatewayinfo['currency']] == 1 ? $gatewayinfo['currency'] : "USD");
		$currency = (in_array($gatewayinfo['currency'], googlecheckout::$supportedCurrencyCodes) ? $gatewayinfo['currency'] : "USD");

		// If testmode is set the url to the test url
		if ($gatewayinfo['testmode']['value'] == "True")
			$this->google_url = $this->google_url_test;

		// If pending Order IDs are passed, add these as a field to be passed
		// back by Google
		if (is_array($orderinfo['pending']))
			$pending = base64_encode(serialize($orderinfo['pending']));

		// Complete the POST URL
		$this->google_url .= $gatewayinfo['username']['value'] . "/checkout";

		// Complete the Button URL
		$this->button['url'] .= $gatewayinfo['username']['value'] . "&w=160&h=43&style=white&variant=text&loc=en_US";

		// Create the cart XML
		$cart =<<<EOT
<?xml version="1.0" encoding="UTF-8"?>
<checkout-shopping-cart xmlns="http://checkout.google.com/schema/2">
  <shopping-cart>
    <items>
      <item>
        <item-name>{$orderinfo['description']}</item-name>
        <item-description></item-description>
        <unit-price currency="{$currency}">{$orderinfo['amount']}</unit-price>
        <quantity>1</quantity>
      </item>
    </items>
    <merchant-private-data>
        <iid>{$orderinfo['iid']}</iid>
        <uid>{$orderinfo['uid']}</uid>
        <coupon>{$orderinfo['coupon']}</coupon>
        <pending>{$pending}</pending>
    </merchant-private-data>
  </shopping-cart>
  <checkout-flow-support>
    <merchant-checkout-flow-support/>
  </checkout-flow-support>
</checkout-shopping-cart>
EOT;
		// Set the cart XML and signature
		$this->hiddenFields['cart'] = base64_encode($cart);
		$this->hiddenFields['signature'] = base64_encode($this->getfingerprint($cart, $gatewayinfo['key']['value']));
	}

	public function getCurrencies() {
		return self::$supportedCurrencyCodes;
	}

	// Creates the cart fingerprint using sha1() one-way hash function with the
	// XML cart data and the merchant key.
	private function getfingerprint($data, $key) {
		$blocksize = 64;

		// Pack the key if it exceeds the block size
		if (strlen($key) > $blocksize)
			$key = pack('H*', sha1($key));

		// Pad the key to the appropriate length with null characters
		$key = str_pad($key, $blocksize, chr(0x00));

		// XOR strings
		$ipad = str_repeat(chr(0x36), $blocksize); // 6's
		$opad = str_repeat(chr(0x5c), $blocksize); // V's

		return pack('H*', sha1(($key^$opad).pack('H*', sha1(($key^$ipad).$data))));
	}


	public function getFields() {
		$i=0;
		foreach ($this->hiddenFields as $key => $value) {
			$fields['hidden'][$i]['name'] = $key;
			$fields['hidden'][$i++]['value'] = $value;
		}

		$fields['postto'] = $this->google_url;
		$fields['button'] = $this->button;

		return $fields;
	}

	public function getSettingFields() {
		return  googlecheckout::$settingFields;
	}

	// Handle the callback procedure
	public function process($postvars, $getvars) {
		// Log data received from the gateway
		$this->logData("googlecheckout", $postvars, "output", false);
		
		// Confirm that the key sent by Google matches our setting key.
		// This prevents just anyone from making fake calls to callback.php
		// pretending to be Google.
		if ($getvars['key'] != $this->gatewayinfo['opt1']['value'])
			return false;

		$this->updateLogStatus("output", true);

		// Evaluate Google's response
		if ($postvars['_type'] == "new-order-notification") {
			// Parse out the invoice id and user id for this payment
			preg_match("/<iid>([0-9]+)<\/iid>/", $postvars['shopping-cart_merchant-private-data'], $invoice);
			preg_match("/<uid>([0-9]+)<\/uid>/", $postvars['shopping-cart_merchant-private-data'], $user);
			preg_match("/<pending>(.+)<\/pending>/", $postvars['shopping-cart_merchant-private-data'], $pending);
			preg_match("/<coupon>(.+)<\/coupon>/", $postvars['shopping-cart_merchant-private-data'], $coupon);

			if (strlen($pending) > 0)
				$trans['pending'] = unserialize(base64_decode($pending));

			// Set the transaction information and return it.
			$trans['amount'] = $postvars['order-total'];
			$trans['currency'] = $postvars['order-total_currency'];
			$trans['uid'] = $user[1];
			$trans['iid'] = $invoice[1];
			$trans['type'] = googlecheckout::$gw;
			$trans['transid'] = $postvars['google-order-number'];
			// Piggy-back the invoice id and pending service ids and coupon temporarily off the message field.
			$trans['message'] = base64_encode(serialize(array("iid" => $invoice[1], "pending" => $trans['pending'], "coupon" => $coupon)));
			$trans['status'] = "pending"; // Must be defined in /inc/config.php under "PAYSTATUS"

			return $trans;
		}
		else {
			// Set the transaction information and return it.
			$trans['type'] = googlecheckout::$gw;
			$trans['transid'] = $postvars['google-order-number'];
			$trans['message'] = "";

			switch ($postvars['new-financial-order-state']) {
				case "CHARGED":
					$trans['status'] = "approved"; // Must be defined in /inc/config.php under "PAYSTATUS"
               		return $trans;
				case "PAYMENT_DECLINED":
					$trans['status'] = "declined"; // Must be defined in /inc/config.php under "PAYSTATUS"
               		return $trans;
				case "CANCELLED":
					$trans['status'] = "voided"; // Must be defined in /inc/config.php under "PAYSTATUS"
               		return $trans;
				case "CANCELLED_BY_GOOGLE":
					$trans['status'] = "voided"; // Must be defined in /inc/config.php under "PAYSTATUS"
               		return $trans;
			}
			return true;
		}
	}
}
?>