<?php

include_once(ROOTWEBDIR . "inc/pdf/class.ezpdf.php");

class InvoicePDF implements InvoiceInterface {
	private $pdf;
	private $company_info;
	private $lines_per_page = 20; // Number of line items allowed per page
	private $max_line_width = 57; // Maximum number of characters allowed per line item (before word-wrap)
	private $background;
	private $page_block;
	
	public function __construct($company_info) {
		$this->pdf = new Cezpdf();
		$this->company_info = $company_info;
		$this->background = $this->company_info['background'];
		
		// Basic setting for this PDF
		$this->pdf = new Cezpdf('a4', 'portrait');
		$this->pdf->setLineStyle(0.5, 'square');
		$this->pdf->ezSetMargins("30", "30", "22", "22");
		$all = $this->pdf->openObject();
		$this->pdf->setStrokeColor(0, 0, 0, 1);
		$this->pdf->closeObject();
		$this->pdf->addObject($all, 'all');
		
		$this->pdf->addPngFromFile($this->background, 0, 0, 596, 842);
		
		// Set the font to use
		$this->pdf->selectFont(ROOTWEBDIR . "inc/pdf/fonts/Helvetica.afm");
		
	}
	
	public function makeInvoice($invoice_data) {
		// Loop through all of the given invoices
		for ($i=0; $i<count($invoice_data); $i++) {
			// Draw each invoice
			$this->drawInvoice($invoice_data[$i], ($i==0 ? true : false));
		}
	}
	
	/**
	 * Returns the current PDF document
	 * 
	 * @return string The PDF document
	 */
	public function fetch() {
		return $this->pdf->ezOutput();
	}
	
	/**
	 * Streams the current PDF document
	 */
	public function stream() {
		$this->pdf->ezStream();
	}
	
	/**
	 * Creates a new PDF page and adds the background image
	 */
	private function newPage() {
		$this->pdf->ezNewPage();
		
		// Add the background for this page
		$this->pdf->addPngFromFile($this->background, 0, 0, 596, 842);
	}
	
	/**
	 * Shortcut for formatting currencies
	 *
	 * @param float $value The value to be formated
	 * @param array $currency The currency to format $value into
	 * @return string The formatted currency value
	 */
	private function format_currency($value, $currency) {	
		return format_currency($value, true, false, true, $currency);
	}
	
	private function countPagesRequired($line_items) {
		$lines = 0;
		$pages = 1;
		
		for ($i=0; $i<count($line_items); $i++) {
			$line = max(1, strlen($line_items[$i]['name'])/$this->max_line_width);
			
			if ($lines + $line > $this->lines_per_page) {
				$pages++;
				$lines = 0;
			}
			else
				$lines += $line;
		}
		
		return $pages;
	}
	
	private function countLines($text) {
		$lines = 0;
		$line_count = 0;
		for ($i=0; $i<strlen($text); $i++) {
			if ($text[$i] == "\n") {
				$line_count = 0;
				$lines++;
			}
			$line_count++;
			
			if ($line_count >= $this->max_line_width) {
				$line_count = 0;
				$lines++;
			}
		}
		return $lines;
	}
	
	/**
	 * Draws the Invoice Header, which is consistent on every page of the invoice.
	 *
	 * @param array $inv An array of invoice data
	 */
	private function drawInvoiceHeader($inv) {
		$ypos = $this->pdf->y;
		
		// Set the company name or logo if provided
		if ($this->company_info['use'] == "logo") {
			// Place the logo on the page
			$this->pdf->ezImage($this->company_info['logo'], 5, 145, '', 'left');
		}
		elseif ($this->company_info['use'] == "name") {
			// Place the company name on the page
			$this->pdf->ezText(utf8_decode($this->company_info['name']), 12, array('justification'=>'left','left'=>15));
			$this->pdf->ezText(utf8_decode($this->company_info['address']), 12, array('justification'=>'left','left'=>15));
		}
		
		// Invoice Date
		$this->pdf->ezSetY($ypos-33);
		$this->pdf->ezText(utf8_decode(_INVOICE_PDF_INVDATE . comfyDate($inv['date_billed'])), 12, array('justification'=>'right'));


		// Tax ID/VATIN (only display if present and tax exists on this invoice)
		if (strlen($this->company_info['tax_id']) > 0 && $inv['tax1_total'] > 0) {
			$this->pdf->ezText(utf8_decode($this->company_info['tax_id']), 12, array('justification'=>'right'));
			$this->pdf->ezSetDy(14); // Adjust for this line
		}
		$this->pdf->ezSetDy(-76);


		// Billing information (name,address, etc.)
		$dy = 24;
		$line = $inv['billing']['fname'] . " " . $inv['billing']['lname'] . "\n";
		if (strlen($inv['billing']['company']) > 0) {
			$line .= $inv['billing']['company'] . "\n";
			$dy+=14;
		}
		$line .= $inv['billing']['address'] . "\n";
		if (strlen($inv['billing']['address2']) > 0) {
			$line .= $inv['billing']['address2'] . "\n";
			$dy+=14;
		}
		$line .= $inv['billing']['city'] . ", " . $inv['billing']['state'] . " " . $inv['billing']['zip'] . " " . $inv['billing']['country'];
		$this->pdf->ezText(utf8_decode($line), 12, array('justification'=>'left', 'left'=>30, 'right'=>200));

		
		// Invoice Number/Customer ID
		$this->pdf->ezSetDy($dy);
		$line = _INVOICE_PDF_INVNUMBER . $inv['id'] . "\n" . _INVOICE_PDF_CUSTOMERID . $inv['client_id'];
		$this->pdf->ezText(utf8_decode($line), 12, array('justification'=>'left', 'left'=>378, 'right'=>22));

		
		// Invoice Summary Line
		$this->pdf->ezSetDy(-68);
		$tdata = array(array('previous_balance'=>$this->format_currency($inv['previous_balance'], $inv['currency']), 'amount_applied'=>$this->format_currency($inv['amount_applied'], $inv['currency']), 'total'=>$this->format_currency($inv['total'], $inv['currency']), 'amount_due'=>$this->format_currency($inv['amount_due'], $inv['currency']), 'date_due'=>comfyDate($inv['date_due'])));
		$this->pdf->ezTable($tdata, array('previous_balance'=>'', 'amount_applied'=>'', 'total'=>'', 'amount_due'=>'', 'date_due'=>'')
				, ''
				, array('showHeadings'=>0, 'fontSize'=>10, 'shaded'=>0, 'showLines'=>0, 'width'=>546, 'xPos'=>'25', 'xOrientation'=>'right', 'cols'=>array(
					'previous_balance'=>array('width'=>112,'justification'=>'center'),
					'amount_applied'=>array('width'=>112,'justification'=>'center'),
					'total'=>array('width'=>112,'justification'=>'center'),
					'amount_due'=>array('width'=>105,'justification'=>'center'),
					'date_due'=>array('width'=>105,'justification'=>'center')
					)
				)
		);
		
		$this->pdf->ezSetDy(-30);
	}
	
	/**
	 * Draws the invoice footer, which is consistent on every page of the invoice
	 *
	 * @param array $inv An array of invoice data
	 */
	private function drawInvoiceFooter($inv) {
		$this->pdf->ezSetY(135);
		$this->pdf->ezText(utf8_decode($this->company_info['terms']), 10, array('justification'=>'left', 'left'=>30, 'right'=>20));
	}
	
	/**
	 * Generates a single invoice, which may appears on one or more pages.
	 *
	 * @param array $inv Invoice data
	 * @param boolean $first_page True of this is the first page being drawn in this PDF document
	 */
	private function drawInvoice($inv, $first_page) {
		
		// Stops the previous invoice's page number, so we can start a new one
		if ($this->page_block >= 0)
			$this->pdf->ezStopPageNumbers(1, 1, $this->page_block); // Stop page numbering
		
		// If this is not the first page, we need to start a new page
		if (!$first_page)
			$this->newPage();
			
		// Starts page numbering, store this page block for reference.
		$this->page_block = $this->pdf->ezStartPageNumbers(572, 738, 12, "left", str_replace(array("[x]","[y]"), array("{PAGENUM}","{TOTALPAGENUM}"), _INVOICE_PDF_PAGE), 1);

		// All pages needed for this invoice
		$pages_needed = $this->countPagesRequired($inv['line_items'], $inv['public_notes']);
		
		// Cycle through all pages required to display all line items for this invoice
		for ($i=0, $j=0; $i<$pages_needed; $i++) {
			
			// Create a new page so we can fit more line items
			if ($i > 0)
				$this->newPage();
			
			// Draw the header which is consistent for all pages
			$this->drawInvoiceHeader($inv);
			
			// Build all line items for this page
			$tdata = array(); // Holds all line item table data
			$lines = 0;
			for ($k=0, $j; $j<count($inv['line_items']); $k++) {
				$line = max(1, (int)(strlen($inv['line_items'][$j]['name'])/$this->max_line_width));
				
				if ($lines + $line > $this->lines_per_page)
					break; // Start a new page
				$lines+=$line;
				
				$tdata[$k] = $inv['line_items'][$j];
				$tdata[$k]['name'] = utf8_decode($tdata[$k]['name']);
				
				// Format the line item price for this currency and tax type
				// Sales is formatted without tax applied to line items
				if ($inv['line_items'][$j]['tax1']['type'] == "sales")
					$tdata[$k]['price'] = $this->format_currency($tdata[$k]['price'], $inv['currency']);
				// VAT and GST include tax in line items ('total' is line item 'price' with tax included)
				else
					$tdata[$k]['price'] = $this->format_currency($tdata[$k]['total'], $inv['currency']);
				
				$j++; // increment that this line item has been added
			}
			
			// Print all line items
			$cols = array('name'=>'', 'price'=>'');
			$this->pdf->ezTable($tdata, $cols, '',
				array('shaded'=>0,
					'showLines'=>0,
					'fontSize'=>10,
					'xPos'=>40,
					'xOrientation'=>'right',
					'width'=>525,
					'cols'=>array(
							'name'=>array('justification'=>'left', 'width'=>420),
							'price'=>array('justification'=>'right', 'width'=>105)
						)
				)
			);
			
			// Add tax line(s) if this is the last page
			if ($i==($pages_needed-1) && $inv['tax1_total'] > 0) {
				$tdata = array();
				$m=0;
				
				// Build tax 1 line
				$tdata[$m]['name'] = utf8_decode($inv['tax1_name'] . " " . $inv['tax1_rate'] . _PERCENTAGE);
				$tdata[$m++]['price'] = $this->format_currency($inv['tax1_total'], $inv['currency']);
				
				if ($inv['tax2_total'] > 0) {
					$tdata[$m]['name'] = utf8_decode($inv['tax2_name'] . " " . $inv['tax2_rate'] . _PERCENTAGE);
					$tdata[$m++]['price'] = $this->format_currency($inv['tax2_total'], $inv['currency']);
				}
				
				$lines+= count($tdata);
				// Evaluate if we need a new page before we can create these line items
				if ($lines > $this->lines_per_page) {
					$lines = count($tdata);
					$this->newPage();
					
					// Draw the header which is consistent for all pages
					$this->drawInvoiceHeader($inv);
				}
				
				// Print the tax line items
				$cols = array('name'=>'', 'price'=>'');
				$this->pdf->ezTable($tdata, $cols, '',
					array('shaded'=>0,
						'showLines'=>0,
						'fontSize'=>10,
						'xPos'=>40,
						'xOrientation'=>'right',
						'width'=>525,
						'cols'=>array(
								'name'=>array('justification'=>'right', 'width'=>420),
								'price'=>array('justification'=>'right', 'width'=>105)
							)
					)
				);
			}
			// Add notes if this is the last page
			if ($i==($pages_needed-1)) {
				$lines += $this->countLines($inv['public_notes']);
				
				// If not enough room on this page, add a new page just for the notes
				if ($lines > $this->lines_per_page) {
					$this->newPage();
					
					// Draw the header which is consistent for all pages
					$this->drawInvoiceHeader($inv);
				}
				$this->pdf->ezSetDy(-20);
				$this->pdf->ezText(utf8_decode($inv['public_notes']), 10, array('justification'=>'left', 'left'=>20, 'right'=>20));
			}
			
			$this->drawInvoiceFooter($inv);
		}
	}
}
?>