<?php
/*******************************************************************************
 @name solusvm_openvz
 @author Tyson Phillips
 @date Oct 14, 2010
 @version 1.0
 @notes For use with Blesta version 2.3.0+
*******************************************************************************/

class solusvm_openvz implements ModuleInterface {
	private static $module;
	private static $server_type = "openvz";

	private $ip;
	private $username;
	private $key;
	private $templates;

	public function __construct() {
		
		// Adds a new solusvm that can be added, edited, or deleted
		self::$module = new Module("solusvm_openvz", false, true, false);

		// Adds the given fields to the database for this module if it has not already
		// been created. WARNING: if the module has already been created with the name
		// given in the Module constructor you must delete the old module before the
		// new one will be created. This may have an adverse effect on services.
		self::$module->addDBField("varchar(255)", "username", "User ID");
		self::$module->addDBField("varchar(255)", "key", "Key");
		self::$module->addDBField("varchar(255)", "serverip", "IP");
		self::$module->addDBField("longtext", "templates", "Templates");
		self::$module->addDBField("datetime", "templates_updated", "Templates Updated");

		// Creates the module, adds the module to the database if the module table does not
		// already exist. WARNING: if a module has already been created and you attempt to
		// add additional fields they WILL NOT BE ADDED, and any refrence to those fields
		// WILL RESULT IN AN ERROR.
		self::$module->createModule();

		// Options for creating a new module row
		// Example: solusvm module may store multiple different servers. The HTML Fields below
		// allow you to create/edit a server for the solusvm module
		// NOTE: The HTML Fields will appear in the order defined here on the module setup/edit pages
		self::$module->addModuleHTMLField("text", "User ID", "username");
		self::$module->addModuleHTMLField("text", "Key", "key");
		self::$module->addModuleHTMLField("text", "IP", "serverip");

		// Module Options while creating a package with this module
		// NOTE: Only one field is accepted of this type and must be named "mstring"
		self::$module->addPackageHTMLField("text", "Node and Plan", "mstring", "^[a-zA-Z0-9]+[|]{1}[a-zA-Z0-9]+", "You must specify a node name and a plan name.");
		
		// This adds a note to be displayed along with the above package field
		self::$module->addPackageNote("mstring", "The name of the node and plan that a VPS created with this package will use, separated by |. (e.g. vm1|plan1)");

		// Specify which field from the module table to display in the drop down and give it a friendly name
		// By default items will be shown by ID and have the friendly name of "Module ID".
		// This appears when adding or editing a service for a user.
		self::$module->setModuleRowSelect("VPS Account", "serverip");

		// These are shown as column names for each service of this type.
		self::$module->addServiceColumn("opt2", "IP Address");
		self::$module->addServiceColumn("pass", "Root Password");
		self::$module->addServiceColumn("user2", "SolusVM Username/Password");

		// These are displayed as tags and are usable when creating/editing a package's welcome e-mail
		self::$module->addServiceTag("serverip", "masterip"); // The master server IP
		self::$module->addServiceTag("user1", "hostname"); // The hostname of the VPS
		self::$module->addServiceTag("user2", "usernamepassword"); // The username and password for SolusVM
		self::$module->addServiceTag("pass", "rootpass"); // The root password for the VPS
		self::$module->addServiceTag("opt2", "ip"); // The virtual machine IP address
		self::$module->addServiceTag("term", "term");

		// These are the fields that may be edited when performing a local edit on this service
		self::$module->addServiceEditField("user1", "Hostname");
		self::$module->addServiceEditField("pass", "Root Password");
		self::$module->addServiceEditField("user2", "SolusVM user/pass");
		self::$module->addServiceEditField("plan", "Package/Term"); // Package/Term
		self::$module->addServiceEditField("dated", "Date Deleted");
		self::$module->addServiceEditField("datep", "Date Suspended");
		self::$module->addServiceEditField("dater", "Date Renews");
		self::$module->addServiceEditField("notes", "Notes");
		
		// Request all templates
		$this->templates = $this->getTemplates(self::$server_type);

		// Fields that are shown to the admins when adding a service that uses this module.
		self::$module->addServiceHTMLField("text", "Hostname", "user1", "^[a-z0-9_%-]+\.[a-z0-9._%-]+\.[a-z]{2,6}$", "The hostname you entered does not appear to be valid. It should be lowercase and in the format: sub.domain.com.");
		self::$module->addServiceHTMLField("select", "Template", "detail[template]", "^[ -~]+$", "The template must not be left empty.");
			if (is_array($this->templates)) {
				for ($i=0; $i<count($this->templates); $i++) {
					self::$module->addServiceHTMLSelectOption("detail[template]", $this->templates[$i], $this->templates[$i]);
				}
			}
		
		// Fields that are shown to the user when adding a service that uses this module.
		self::$module->addUserServiceHTMLField("text", "Hostname", "user1", "^[a-z0-9_%-]+\.[a-z0-9._%-]+\.[a-z]{2,6}$", "The hostname you entered does not appear to be valid. It should be lowercase and in the format: sub.domain.com.");
		self::$module->addUserServiceHTMLField("select", "Template", "detail[template]", "^[ -~]+$", "The template must not be left empty.");
			if (is_array($this->templates)) {
				for ($i=0; $i<count($this->templates); $i++) {
					self::$module->addUserServiceHTMLSelectOption("detail[template]", $this->templates[$i], $this->templates[$i]);
				}
			}

		// Add management tabs for Clients/Admins to manage services of this module type
		self::$module->addAction("Overview", array($this, "getOverview"));
	}

	/**
	 * Initialize an API request based on the given service information
	 *
	 * @param array $serviceInfo The service being examined
	 */
	private function initAction($serviceInfo) {
		$conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
		$data = self::$module->getRows($conditions);
		$data = $data[0];
		
		$this->username = $data['username'];
		$this->key = $data['key'];
		$this->ip = $data['serverip'];
	}

	/**
	 * Create the Overview tab of the manage serive section
	 *
	 * @param array $serviceInfo The service being examined
	 * @return string HTML to send to output for this tab
	 */
	public function getOverview($serviceInfo) {
		global $mysqldb;
		
		$this->initAction($serviceInfo);
		
		$t = new Template(dirname(__FILE__) . "/templates/get_overview.pdt");
		
		if (isset($_GET['do']) && $serviceInfo['datep'] == "0000-00-00" && $serviceInfo['dated'] == "0000-00-00") {
			$message = "The action was sent";
			
			switch ($_GET['do']) {
				case "reboot":
					$this->rebootVPS($serviceInfo['opt1']);
					break;
				case "shutdown":
					$this->shutdownVPS($serviceInfo['opt1']);
					break;
				case "boot":
					$this->bootVPS($serviceInfo['opt1']);
					break;
				case "edit":
					if (!empty($_POST)) {
						$fields = self::$module->getUserServiceHTMLFields();
						foreach ($_POST as $key => $value) {
							for ($i=0; $i<count($fields); $i++) {
								if ($fields[$i]['dbFieldName'] == $key) {
									// test regex for validity
									if (!preg_match("/" . $fields[$i]['regex'] . "/", $value)) {
										$error = $fields[$i]['errorMessage'];
										break 2;
									}
									
									break; // on to the next field
								}
							}
						}
						
						if (!isset($error)) {
							self::$module->updateService($serviceInfo['id'], $_POST);
							
							$message = "The information was successfully updated";
							
							// Update visually, all service fields
							foreach ($_POST as $key => $value) {
								if (isset($serviceInfo[$key]))
									$serviceInfo[$key] = $value;
							}
						}
					}
					break;
			}
			
			if ($error)
				$t->set("error", $error);
			elseif ($message)
				$t->set("message", $message);
		}
		
		$postto = preg_replace("/ajax=([^&]*)&?/i", "", $_SERVER['REQUEST_URI']);
		
		// Get status of VPS
		$status = $this->getStatus($serviceInfo['opt1']);

		$vps = array(
			"running" => false,
			"disabled" => false,
			"primary_ip_address" => $serviceInfo['opt2']
		);
		
		switch ($status) {
			case "online":
				$vps['running'] = true;
				break;
			case "offline":
			case "disabled":
				$vps['disabled'] = true;
				break;			
			default:
				break;
		}
			
		$t->set("vps", $vps);
		
		$t->set("service", $serviceInfo);
		$t->set("postto", $postto);
		$t->set("template_dir", "/" . str_replace(ROOTWEBDIR, "", dirname(__FILE__) . "/templates"));
		
		return $t->fetch();
	}

	/**
	 * Returns the module object, this function MUST be present
	 *
	 * @return Module the module this class initalizes and modifies
	 */
	public function getModule() {
		return self::$module;
	}

	/*******************************************************************************
	********************************************************************************
	* This is the MAGIC function. This function automatically creates a service    *
	* for a user.                                                                  *
	*                                                                              *
	* IT MUST BE DEFINED HERE OR AN E-MAIL WILL BE SENT TO ALL SUBSCRIBED ADMINS   *
	* NOTIFYING THEM THAT THE SERVICE MUST BE MANUALLY CREATED.                    *
	*                                                                              *
	* IF THIS FUNCTION IS DEFINED AND RETURNS TRUE IT IS ASSUMED THAT IT PERFORMS  *
	* THE NECCESSARY ACTIONS TO AUTOMATICALLY ACTIVATE THE SERVICE.                *
	********************************************************************************
	*******************************************************************************/

	public function promodule($action, $serviceInfo, $packageInfo) {
		global $mysqldb;
		
		// packageInfo comes from packagesetinfos() function
		// The name of the node and the plan name
		$nodeplan = explode("|", $packageInfo[4][2]);
		$node = $nodeplan[0];
		$plan = $nodeplan[1];
	
		$conditions = array(array("field" => "id", "value" => $serviceInfo['mid']));
		$data = self::$module->getRows($conditions);
		$data = $data[0];
		
		$this->username = $data['username'];
		$this->key = $data['key'];
		$this->ip = $data['serverip'];
	
		switch ($action) {
			case "add":
				if (DEMO)
					return $serviceInfo;
				
				// Get Client data
				$client = auserinfo($mysqldb, $serviceInfo['uid']);
				
				$vars = array();				
				$vars['hostname'] = $serviceInfo['user1'];
				$vars['template'] = $serviceInfo['detail']['template'];
				$vars['plan'] = $plan;
				$vars['username'] = $serviceInfo['uid'];
				$vars['fname'] = $client[2];
				$vars['lname'] = $client[3];
				$vars['email'] = $client[11];
				$vars['password'] = "unknown";
				$vars['type'] = self::$server_type;
				$vars['ips'] = 1;
				$vars['node'] = $node;
				
				$result = $this->addVPS($vars);
				
				if ($result) {
					// Password is passed back from addVPS and changed when a new Client is created
					$serviceInfo['user2'] = $vars['username'] . "/" . $vars['password'];					
					$serviceInfo['pass'] = $result['rootpassword'];
					$serviceInfo['opt1'] = $result['vserverid'];
					$serviceInfo['opt2'] = $result['mainipaddress'];
					return $serviceInfo;
				}
				return false;
				break;
			case "remove":
				if (DEMO)
					return true;
				return $this->removeVPS($serviceInfo['opt1']);
				break;
			case "suspend":
				if (DEMO)
					return true;
				return $this->suspendVPS($serviceInfo['opt1']);
				break;
			case "unsuspend":
				if (DEMO)
					return true;
				return $this->unsuspendVPS($serviceInfo['opt1']);
				break;
		}
	
		return false;
	}


	/* ------------------------------------------------------------------------------------------------
   The following are support functions for this module
	--------------------------------------------------------------------------------------------------*/
	
	/**
	 * Get all templates of a given type
	 *
	 * @param string $type The type of templates to get (openvz, xen, xen hvm)
	 */
	private function getTemplates($type) {
		static $templates = null;
		
		if ($templates == null) {
			// Fetch templates
			$data = self::$module->getRows();
			$data = $data[0];
			
			$templates = array();
			// If templates are stale, update them
			if ($data['templates_updated'] == null || time() > strtotime($data['templates_updated'] . " +1 day")) {
				$this->username = $data['username'];
				$this->key = $data['key'];
				$this->ip = $data['serverip'];
				
				$params = array();
				$response = array();
				
				$params['type'] = $type;
				$params['action'] = "listtemplates";
				
				$response = $this->call("POST", $params);

				
				if ($response['status'] == "success") {
					// Parse templates
					$templates = explode(",", $response['templates']);
					
					if (!empty($templates)) {
						self::$module->updateDBFields("templates", base64_encode(serialize($templates)), $data['id']);
						self::$module->updateDBFields("templates_updated", date("Y-m-d H:i:s"), $data['id']);
					}
					else {
						$templates = unserialize(base64_decode($data['templates']));
					}
				}
			}
			else
				$templates = unserialize(base64_decode($data['templates']));
		}
		return $templates;
	}
	
	/**
	 * Get status of VPS
	 *
	 * @param $id The virtual server id
	 * @return string The status message response
	 */
	private function getStatus($id) {
		$params = array();
		$response = array();
		
		$params['vserverid'] = $id;
		$params['action'] = "vserver-status";
		
		$response = $this->call("POST", $params);
		
		$status = ($response['status'] == "success" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		$message = "";
		if ($status)
			$message = $response['statusmsg'];
			
		return $message;
	}
	
	/**
	 * Boot a VPS
	 *
	 * @param int $id The virtual server ID to boot
	 * @return boolean True on success, false otherwise
	 */
	private function bootVPS($id) {
		$params = array();
		$response = array();
		
		$params['vserverid'] = $id;
		$params['action'] = "vserver-boot";
		
		$response = $this->call("POST", $params);
		
		$status = ($response['status'] == "success" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;		
	}

	/**
	 * Reboot a VPS
	 *
	 * @param int $id The virtual server ID to reboot
	 * @return boolean True on success, false otherwise
	 */
	private function rebootVPS($id) {
		$params = array();
		$response = array();
		
		$params['vserverid'] = $id;
		$params['action'] = "vserver-reboot";
		
		$response = $this->call("POST", $params);
		
		$status = ($response['status'] == "success" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;		
	}
	
	/**
	 * Shutdown a VPS
	 *
	 * @param int $id The virtual server ID to shutdown
	 * @return boolean True on success, false otherwise
	 */
	private function shutdownVPS($id) {
		$params = array();
		$response = array();
		
		$params['vserverid'] = $id;
		$params['action'] = "vserver-shutdown";
		
		$response = $this->call("POST", $params);
		
		$status = ($response['status'] == "success" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;		
	}	
	
	/**
	 * Suspend a VPS
	 *
	 * @param int $id The virtual server ID to suspend
	 * @return boolean True on success, false otherwise
	 */
	private function suspendVPS($id) {
		$params = array();
		$response = array();
		
		$params['vserverid'] = $id;
		$params['action'] = "vserver-suspend";
		
		$response = $this->call("POST", $params);
		
		$status = ($response['status'] == "success" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;
	}
	
	/**
	 * Unsuspend a VPS
	 *
	 * @param int $id The virtual server ID to suspend
	 * @return boolean True on success, false otherwise
	 */
	private function unsuspendVPS($id) {
		$params = array();
		$response = array();
		
		$params['vserverid'] = $id;
		$params['action'] = "vserver-unsuspend";
		
		$response = $this->call("POST", $params);
		
		$status = ($response['status'] == "success" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;
	}
	
	/**
	 * Terminate a VPS
	 *
	 * @param int $id The virtual server ID of the VPS to remove
	 * @return boolean True on success, false otherwise
	 */
	private function removeVPS($id) {
		$params = array();
		$response = array();
		
		$params['vserverid'] = $id;
		$params['deleteclient'] = "true";
		$params['action'] = "vserver-terminate";
		
		$response = $this->call("POST", $params);
		
		$status = ($response['status'] == "success" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		return $status;
	}	
	
	/**
	 * Create a new client
	 *
	 * @return boolean True on success, false otherwise
	 */
	private function createClient(&$vars) {
		$params = array();
		$params['username'] = $vars['username'];
		$params['password'] = $this->makePassword();
		$params['email'] = $vars['email'];
		$params['firstname'] = $vars['fname'];
		$params['lastname'] = $vars['lname'];
		$params['action'] = "client-create";
		
		$response = $this->call("POST", $params);
		
		if ($response['status'] != "success") {
			// Client could not be created
			self::$module->logData($response, "output", false);
			return false;
		}
		
		// Pass back new password
		$vars['password'] = $params['password'];
		return true;
	}

	/**
	 * Create a new VPS
	 *
	 * @return mixed Array on success, false otherwise
	 */
	private function addVPS(&$vars) {
		// Check if client exists
		$response = array();
		$client = array();
		
		$client['username'] = $vars['username'];
		$client['action'] = "client-checkexists";
		$response = $this->call("POST", $client);
		
		if ($response['status'] != "success") {
			// Client does not exist, create one
			if (!$this->createClient($vars))
				return false;
		}
		
		// Create the VPS
		$params = array();
		$params['type'] = $vars['type'];
		$params['node'] = $vars['node'];
		$params['hostname'] = $vars['hostname'];
		$params['password'] = $this->key;
		$params['username'] = $vars['username'];
		$params['plan'] = $vars['plan'];
		$params['template'] = $vars['template'];
		$params['ips'] = $vars['ips'];
		$params['action'] = "vserver-create";
		
		$response = $this->call("POST", $params);
		
		$status = ($response['status'] == "success" ? true : false);
		
		self::$module->logData($response, "output", $status);
		
		if ($status)
			return $response;
		return false;
	}
	
	/**
	 * Make the call
	 *
	 * @return array The key => value pairs of the returned response
	 */
	private function call($method="POST", $params) {
		self::$module->logData($method . " " . $this->ip . "\n" . ($params ? print_r($params, true) : ""), "input", true);
	
		$params['id'] = $this->username;
		$params['key'] = $this->key;

		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, "https://" . $this->ip . ":5656/api/admin/command.php");
		curl_setopt($ch, CURLOPT_HEADER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
		curl_setopt($ch, CURLOPT_POST, true);
		
		$response = array();
		$response = curl_exec($ch);

		curl_close($ch);
		
		$match = array();
		preg_match_all('/<(.*?)>([^<]+)<\/\\1>/i', $response, $match);
		
		$result = array();
		foreach ($match[1] as $key => $value) {
			$result[$value] = $match[2][$key];
		}
		
		return $result;
	}
	
	// Creates a password that is 8 characters long
	private static function makePassword() {
		$pass = "";
		
		// Add 8-random characters
		$chars = array('a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t',
		'u','v','w','x','y','z','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R',
		'S','T','U','V','W','X','Y','Z','0','1','2','3','4','5','6','7','8','9');
		$count = count($chars) - 1;
	
		for ($i=0; $i<8; $i++)
			$pass = $chars[rand(0, $count)] . $pass;
	 
		return $pass;
	}	
}
?>